// Copyright (C) 2003 Kevin Atkinson (kevin.med@atkinson.dhs.org) under 
// the GNU GPL version 2.

typedef unsigned char byte;

// static inline byte min(byte x, byte y) {return x < y ? x : y;}
// static inline byte max(byte x, byte y) {return x > y ? x : y;}

// static inline short min(short x, short y) {return x < y ? x : y;}
// static inline short max(short x, short y) {return x > y ? x : y;}

static inline int min(int x, int y) {return x < y ? x : y;}
static inline int max(int x, int y) {return x > y ? x : y;}

#define REGPARM __attribute__ (( regparm(3) ))

#include <stddef.h>

struct MutableRow {
  byte * data;
  int pitch;
  MutableRow() : data(0) {}
  MutableRow(byte * d, int p) : data(d), pitch(p) {}
};

struct ConstRow {
  const byte * data;
  int pitch;
  ConstRow() : data(0) {}
  ConstRow(MutableRow o) : data(o.data), pitch(o.pitch) {}
  ConstRow(byte * d, int p) : data(d), pitch(p) {}
};

#define M_SEARCH 0
#define M_MEDIAN 4

#define S_DENOISE 0
#define S_MARK    1
#define S_MAP     2

struct Parms {

  size_t width, height;
  size_t size;

  byte p1, p2;
  short pwidth, pheight;
  int y_next; // 1 if not interlaces, 2 if
  byte mthres;
  int mwidth, mheight;
  byte mp;
  int mode, show;
  byte mark_v;
  bool show_chroma;

  Parms() 
    : p1(24), p2(12), pwidth(16), pheight(5), y_next(2),
      mthres(16), mwidth(7), mheight(5), mp(11),
      mark_v(255), show_chroma(false) {}
};

struct Border;
extern size_t border_size;

struct Borders {
  Border * data;
  Border * end;
  Borders() : data(0), end(0) {}
  REGPARM void setup(size_t);
  void clear() {end = data;}
};

typedef void Exec(ConstRow p, 
                  ConstRow c, byte * c_noise, byte * c_motion,
                  ConstRow n, byte * n_motion,
                  MutableRow o, const Parms &);

extern "C" {
  REGPARM void find_sizes(Borders & bds, byte * noise, const Parms &);

  REGPARM void mark_noise(Borders & bds, byte * noise, const Parms &);

  REGPARM void memzero(void *, size_t s);

  void find_motion(ConstRow p, ConstRow c, 
                   byte * motion, const Parms &);
  Exec cond_median;
  
  void motion_denoise(byte * moving, const Parms &);

  void find_outliers(ConstRow p, ConstRow c, ConstRow n, 
                     byte * o, const Parms &);

  void mark_motion(ConstRow p, byte * p_noise,
                   ConstRow c, byte * c_noise, byte * c_motion,
                   const Parms &);

  Exec remove_outliers;
  Exec mark_outliers;
  Exec map_outliers;
}



